/* Gimp Plugin to classify parts of an image */

#include <stdlib.h>
#include <libgimp/gimp.h>

static void query (void);
static void run (char* name,
				 int nb_params,
				 GimpParam* params,
				 int* nb_return_vals,
				 GimpParam** return_vals);

GimpPlugInInfo PLUG_IN_INFO =
{
   NULL,		/*  init_proc	*/
   NULL,		/*  quit_proc	*/
   query,		/*  query_proc	*/
   run,			/*  run_proc	*/
};

MAIN()

static void
query(void)
{
   static GimpParamDef args[] =
   {
	  {GIMP_PDB_INT32, "run_mode", "Interactive, non-interactive"},
	  {GIMP_PDB_IMAGE, "image_id", "Input image"},
	  {GIMP_PDB_DRAWABLE, "drawable_id", "Input drawable"},
//	  {GIMP_PDB_INT32, "nb_classes", "Number of classes to find"}
   };

   static GimpParamDef return_vals[] = {
	  {GIMP_PDB_IMAGE, "image", "Output image"},
   };

   static int nb_args =  sizeof (args) / sizeof (args[0]);
   static int nb_return_vals = sizeof (return_vals) / sizeof (return_vals[0]);

   gimp_install_procedure
	  ("plug_in_isodata",
	   "Computes non-supervised classification of an image.",
	   "Computes non-supervised classification of an image"
	   ", given the number of classes.",
	   "tharibo",
	   "tharibo",
	   "2001",
	   "<Image>/Filters/Classification/Isodata...>",
	   "GRAY",
	   GIMP_PLUGIN,
	   nb_args, nb_return_vals, args, return_vals);
}

static void isodata(GimpDrawable drawable, gint32 nb_classes);

static void
run(char* name,
	int nb_params,
	GimpParam* param,
	int* nb_return_vals,
	GimpParam** return_vals)
{
   static GimpParam values[2];
   GimpDrawable *drawable;
   gint32 image, out_image;
   GimpRunModeType run_mode;
   GimpPDBStatusType status = GIMP_PDB_SUCCESS;
   gint32 nb_classes;

   run_mode = param[0].data.d_int32;
   image = param[1].data.d_int32;
   drawable = gimp_drawable_get(param[2].data.d_drawable);

   if(run_mode == GIMP_RUN_INTERACTIVE)
   {
   }
   else if(run_mode == GIMP_RUN_NONINTERACTIVE)
   {
	  if(nb_params != 4)
	  {
		 status = GIMP_PDB_CALLING_ERROR;
	  }
	  else
	  {
		 // Gets the number of classes
		 //FIXME: nb_classes = param[3].data.d_int32;
		 nb_classes = 3;
	  }
   }

   if (status == GIMP_PDB_SUCCESS)
   {
	  /*  Make sure that the drawable is gray */
	  if(gimp_drawable_is_gray(drawable->id))
	  {
		 isodata(*drawable, nb_classes);
		 /*(void)gimp_display_new(out_image);
		 if (run_mode != GIMP_RUN_NONINTERACTIVE)
		 gimp_displays_flush();*/
	  }
	  if(gimp_drawable_is_rgb(drawable->id))
	  {
		 status = GIMP_PDB_EXECUTION_ERROR;
	  }
	  if(gimp_drawable_is_indexed(drawable->id))
	  {
		 status = GIMP_PDB_EXECUTION_ERROR;
	  }
	  gimp_drawable_detach (drawable);
   }

   if(image != -1)
   {
	  *nb_return_vals = 2;
	  values[0].type = GIMP_PDB_STATUS;
	  values[0].data.d_status = status;
	  values[1].type = GIMP_PDB_IMAGE;
	  values[1].data.d_image = out_image;
   }
}

static void
isodata(GimpDrawable drawable, gint32 nb_classes)
{
   GimpDrawable* output_drawable;
   GimpPixelRgn* region;
   GimpPixelRgn* output_region;
   double moyennes[nb_classes];
   gboolean stationnaire;
   guint32 ligne, colonne;
   gint32 output_image_id;
   gint32 output_layer_id;
   gpointer pr;
   int i;

   /* Creates the output image */
   output_image_id = gimp_image_new(drawable.width,
									drawable.height,
									GIMP_GRAY);
   /* Creates a layer on the image */
   output_layer_id = gimp_layer_new(output_image_id,
									"Fond",
									drawable.width,
									drawable.height,
									GIMP_GRAY_IMAGE,
									100.0,		/* opacity */
									GIMP_NORMAL_MODE);
   /* Add the layer to the image*/
   gimp_image_add_layer(output_image_id, output_layer_id, 0);

   /* Create the drawable */
   output_drawable = gimp_drawable_get(output_layer_id);

   /* Cration de la rgion d'entre */
   gimp_pixel_rgn_init(region, &drawable,
					   0, 0,
					   drawable.width, drawable.height,
					   FALSE, FALSE);

   /* Cration de la rgion de sortie */
   gimp_pixel_rgn_init(output_region, output_drawable,
					   0, 0,
					   output_drawable->width, output_drawable->height,
					   TRUE, TRUE);

   /* Initialisation des moyennes : on divise l'ensemble des niveaux
      de gris par le nombre de classes */
   for(i = 0; i < nb_classes; i++)
	  moyennes[i] = i * (256 / nb_classes);

   stationnaire = FALSE;
   while(!stationnaire)
   {
	  /* Parcours de l'image */
	  for(pr = gimp_pixel_rgns_register(2, &region, &output_region);
		  pr != NULL;                                
		  pr = gimp_pixel_rgns_process(pr))
	  {
		 /* Let's process the current tile */
		 for(ligne = 0; ligne < region->h; ligne++)
		 {
			for(colonne = 0; colonne < region->w; colonne++)
			{
			   int indexPixel = ligne * region->rowstride
				  + colonne * region->bpp;
			   double distance_min = 256;
			   guchar classe;

			   /* Trouver la classe de l'chantillon courant */
			   for(i = 0; i < nb_classes; i++)
			   {
				  double distance =
					 abs(region->data[indexPixel] - moyennes[i]);
				  if(distance < distance_min)
				  {
					 distance_min = distance;
					 classe = (guchar)i;
				  }
			   }

			   /* Colorier l'chantillon de la couleur de sa classe */
			   output_region->data[indexPixel] =
				  classe * (256 / nb_classes);
			}
		 }
	  }                         
   }
}
